from vndbapi.core import *
from vndbapi.const import *
from datetime import datetime
import json

def login(username: str, password: str) -> str or dict:
    msg = sendData("login", f'{{"protocol":1,"client":"{CLIENTNAME}","clientver":{CLIENTVER},"username":"{username}","password":"{password}"}}')
    if "error " not in msg:
        return msg
    else:
        return json.loads(msg)

def dbstats() -> dict:
    return json.loads(sendData("dbstats").strip("dbstats "))

def get(typ: str, filters: str, flag:str, page: int=-1, results: int=-1) -> dict:
    cmd = f"get {typ} {flag} {filters}"
    if page != -1:
        jsn = f'{{"page": {page}, "results": {results}}}'
    else:
        jsn = ""
    res = sendData(cmd, jsn).strip("results ")
    return json.loads(res)

def getTempl(iam: str,filterslist: list, flag: str, page: int, results: int) -> dict:
    namecap = iam.capitalize()
    finalfilter = "("
    for item in filterslist:
        chk = bool([itm for itm in disallowedfilters[iam] if(itm in item)]) # check if at least one of the disallowed filters is in the string.
        if chk:
            raise InvalidFilterException(f'"{item}" is not a valid filter for "get{namecap}"')
        finalfilter += item
        finalfilter += " and "
    finalfilter = finalfilter[:-5]
    finalfilter += ")"
    if DBG: print("[DBG]", finalfilter)
    return get(iam, finalfilter, flag, page, results)

def getVn(filterslist: list, flag: str=flags.BASIC, page: int=1, results: int=10) -> dict:
    return getTempl("vn", filterslist, flag, page, results)

def getRelease(filterslist: list, flag: str=flags.BASIC, page: int=1, results: int=10) -> dict:
    return getTempl("release", filterslist, flag, page, results)

def getProducer(filterslist: list, flag: str=flags.BASIC, page: int=1, results: int=10) -> dict:
    return getTempl("producer", filterslist, flag, page, results)

def getCharacter(filterslist: list, flag: str=flags.BASIC, page: int=1, results: int=10) -> dict:
    return getTempl("character", filterslist, flag, page, results)
    
def getStaff(filterslist: list, flag: str=flags.BASIC, page: int=1, results: int=10) -> dict:
    return getTempl("staff", filterslist, flag, page, results)

def getUser(filterslist: list, flag: str=flags.BASIC, page: int=1, results: int=10) -> dict:
    return getTempl("user", filterslist, flag, page, results)
        
def getUlistLabels(uid: int=0) -> dict:
    return get("ulist-labels", f"(uid={uid})", "basic")

def getUlist(label: int, flag: str=flags.ALLUL, uid: int=0) -> dict:
    return get("ulist", f"(uid={uid} and label={label})", flag)

def setUlist(vnid: int,notes: str=None, started: bool=None, finished: bool=None, vote: int=None, labels: list=None) -> str:
    if not isinstance(notes, str):
        if notes is not None:
            raise InvalidTypeException(f'"{notes}" is not a string. "notes" in "setUlist" requires a string (or None).')
    if not isinstance(started, str):
        if started is not None:
            raise InvalidTypeException(f'"{started}" is not a string. "started" in "setUlist" requires a string (or None).')
    else:
        try:
            datetime.strptime(started, '%Y-%m-%d')
        except ValueError:
            raise InvalidTypeException(f'"{started}" is not a valid date (yyyy-mm-dd). "started" in "setUtils" requires a valid date (or None).')
    if not isinstance(finished, str):
        if finished is not None:
            raise InvalidTypeException(f'"{finished}" is not a string. "finished" in "setUlist" requires a string (or None).')
    else:
        try:
            datetime.strptime(finished, '%Y-%m-%d')
        except ValueError:
            raise InvalidTypeException(f'"{finished}" is not a valid date (yyyy-mm-dd). "finished" in "setUtils" requires a valid date (or None).')
    if not isinstance(vote, int) or not (10 <= vote <= 100):
        if vote is not None:
            raise InvalidTypeException(f'"{vote}" is not an integer in the range of 10-100. "vote" in "setUlist" requires a valid integer (or None).')
    if not isinstance(labels, list):
        if labels is not None:
            raise InvalidTypeException(f'"{labels}" is not a list. "labels" in "setUlist" requires a list (or None).')
    
    if notes is None and started is None and finished is None and vote is None and labels is None:
        return sendData(f"set ulist {vnid}")
    else:
        finaljson = "{"
        if notes is not None:
            finaljson += f'"notes":"{notes}",'
        if started is not None:
            finaljson += f'"started":"{started}",'
        if finished is not None:
            finaljson += f'"finished":"{finished}",'
        if vote is not None:
            finaljson += f'"vote":{vote},'
        if labels is not None:
            finaljson += f'"labels":{labels},'
        finaljson = finaljson[:-1]
        finaljson += "}"
        if DBG: print("[DBG]", finaljson)

        return sendData(f"set ulist {vnid}", finaljson)


