from socket import socket, AF_INET, SOCK_STREAM, gethostbyname
from vndbapi.const import *
import json
try:
    from ssl import wrap_socket
except ImportError or ImportWarning:
    nossl = True
else:
    nossl = False

FORCENOSSL = False

if nossl or FORCENOSSL:
    if DBG: print("[DBG] Not using SSL")
    SOCKET = socket(AF_INET, SOCK_STREAM)
    IP = gethostbyname("api.vndb.org")
    SOCKET.connect((IP, 19534))
else:
    if DBG: print("[DBG] Using SSL")
    SOCKET = wrap_socket(socket(AF_INET, SOCK_STREAM), server_side=False)
    IP = gethostbyname("api.vndb.org")
    SOCKET.connect((IP, 19535))

def sendData(command: str, jsn: str="") -> str:
    if DBG: print("[DBG]", command, jsn)
    SOCKET.send(command.encode() + " ".encode() + jsn.encode() + EOT.encode())
    finmsg = b""
    while EOT.encode() not in finmsg:
        finmsg += SOCKET.recv(1024)
    retval = finmsg.decode().strip(EOT)
    if "error" in retval:
        retval = retval.strip("error ")
        retval = json.loads(retval)
        if retval["id"] == "throttled":
            raise ConnectionThrottledException(f'Connection throttled, Reason: {retval["type"]}. You have to wait another {retval["fullwait"]} seconds.')
        else:
            raise APIException(f'"{retval["id"]}"-Error: {retval["msg"]}')
    return retval

class Error(Exception):
    def __init__(self, message):
        self.message = message

class InvalidOperatorException(Error):
    pass

class InvalidTypeException(Error):
    pass

class InvalidFilterException(Error):
    pass

class APIException(Error):
    pass

class ConnectionThrottledException(Error):
    pass

class UnsupportedOSException(Error):
    pass
    
