from vndbapi.core import InvalidOperatorException, InvalidTypeException
from string import ascii_lowercase
from datetime import datetime

### VN

def vnid(op: str, vvnid: int) -> str:
    if op not in ["=", "!=", ">", ">=", "<", "<="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "vnid".')
    if not isinstance(vvnid, (list, int)):
        raise InvalidTypeException(f'"{vvnid}" is neither an integer nor a list. "vnid" requires an integer or a list.')
    return f"id{op}{vvnid}"

def title(op: str, vtitle: str) -> str:
    if op not in ["=", "!=", "~"]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "title".')
    if not isinstance(vtitle, str):
        raise InvalidTypeException(f'"{vtitle}" is not a string. "title" requires a string.')
    return f'title{op}"{vtitle}"'

def original(op: str, voriginal: str) -> str:
    if op not in ["=", "!=", "~"]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "original".')
    if not isinstance(voriginal, str):
        raise InvalidTypeException(f'"{voriginal}" is not a string. "original" requires a string.')
    return f'original{op}"{voriginal}"'

def firstchar(op: str, vfirstchar: str) -> str:
    vfirstchar = vfirstchar.lower()
    AtoZ = ascii_lowercase[:26]
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "firstchar".')
    if vfirstchar not in AtoZ:
        raise InvalidTypeException(f'"{vfirstchar}" is not a single character from a to z. "firstchar" requires a valid string/character.')
    return f'firstchar{op}"{vfirstchar}"'

def released(op: str, vreleased: str) -> str:
    if op not in ["=", "!=", ">", ">=", "<", "<="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "released".')
    if not isinstance(vreleased, str):
        raise InvalidTypeException(f'"{vreleased}" is not a valid string (yyyy-mm-dd). "released" requires a valid string.')
    try:
        datetime.strptime(vreleased, '%Y-%m-%d')
    except ValueError:
        raise InvalidTypeException(f'"{vreleased}" is not a valid date (yyyy-mm-dd). "released" requires a valid date.')
    return f'released{op}"{vreleased}"'

def platforms(op: str, vplatforms: str) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "platforms".')
    if not isinstance(vplatforms, (list, str)):
        raise InvalidTypeException(f'"{vplatforms}" is neither a string nor a list. "platforms" requires a string or a list.')
    if isinstance(vplatforms, str):
        platforms = f'"{platforms}"'
    return f"platforms{op}{platforms}"

def languages(op: str, vlanguages: str) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "languages".')
    if not isinstance(vlanguages, (list, str)):
        raise InvalidTypeException(f'"{vlanguages}" is neither a string nor a list. "languages" requires a string or a list.')
    if isinstance(vlanguages, str):
        vlanguages = f'"{vlanguages}"'
    return f"languages{op}{vlanguages}"

def orig_lang(op: str, vorig_lang: str) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "orig_lang".')
    if not isinstance(vorig_lang, (list, str)):
        raise InvalidTypeException(f'"{vorig_lang}" is neither a string nor a list. "orig_lang" requires a string or a list.')
    if isinstance(vorig_lang, str):
        vorig_lang = f'"{vorig_lang}"'
    return f"orig_lang{op}{vorig_lang}"

def search(op: str, vsearch: str) -> str:
    if op not in ["~"]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "search".')
    if not isinstance(vsearch, str):
        raise InvalidTypeException(f'"{vsearch}" is not a string. "search" requires a string.')
    return f'search{op}"{vsearch}"'

def tags(op: str, vtags: int) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "tags".')
    if not isinstance(vtags, (int, list)):
        raise InvalidTypeException(f'"{vtags}" is neither an integer nor a list. "tags" requires an integer or a list.')
    return f"tags{op}{vtags}"

### RELEASE

def vn(op: str, vvn: int) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "vn".')
    if not isinstance(vvn, (int, list)):
        raise InvalidTypeException(f'"{vvn}" is neither an integer nor a list. "vn" requires an integer or a list.')
    return f"vn{op}{vvn}"

def producer(op: str, vproducer: int) -> str:
    if op not in ["="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "producer".')
    if not isinstance(vproducer, int):
        raise InvalidTypeException(f'"{vproducer}" is not an integer. "producer" requires an integer.')
    return f"producer{op}{vproducer}"

def patch(op: str, vpatch: bool) -> str:
    if op not in ["="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "patch".')
    if not isinstance(vpatch, bool):
        raise InvalidTypeException(f'"{vpatch}" is not a boolean. "patch" requires a boolean.')
    return f"patch{op}{vpatch}"

def freeware(op: str, vfreeware: bool) -> str:
    if op not in ["="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "freeware".')
    if not isinstance(vfreeware, bool):
        raise InvalidTypeException(f'"{vfreeware}" is not a boolean. "freeware" requires a boolean.')
    return f"freeware{op}{vfreeware}"

def doujin(op: str, vdoujin: bool) -> str:
    if op not in ["="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "doujin".')
    if not isinstance(vdoujin, bool):
        raise InvalidTypeException(f'"{vdoujin}" is not a boolean. "doujin" requires a boolean.')
    return f"doujin{op}{vdoujin}"

def vntype(op: str, vvntype: str) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "vntype".')
    if not isinstance(vvntype, str):
        raise InvalidTypeException(f'"{vvntype}" is not a string. "vntype" requires a string.')
    return f'type{op}"{vvntype}"'

def gtin(op: str, vgtin: int) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "gtin".')
    if not isinstance(vgtin, int):
        raise InvalidTypeException(f'"{vgtin}" is not an integer. "gtin" requires an integer.')
    return f"gtin{op}{vgtin}"

def catalog(op: str, vcatalog: str) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "catalog".')
    if not isinstance(vcatalog, str):
        raise InvalidTypeException(f'"{vcatalog}" is not a string. "catalog" requires a string.')
    return f'catalog{op}"{vcatalog}"'

### PRODUCER

def name(op: str, vname: str) -> str:
    if op not in ["=", "!=", "~"]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "name".')
    if not isinstance(vname, str):
        raise InvalidTypeException(f'"{vname}" is not a string. "name" requires a string.')
    return f'name{op}"{vname}"'

### CHARACTER

def traits(op: str, vtraits: int) -> str:
    if op not in ["=", "!="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "traits".')
    if not isinstance(vtraits, (int, list)):
        raise InvalidTypeException(f'"{vtraits}" is neither an integer nor a list. "traits" requires an integer or a list.')
    return f"traits{op}{vtraits}"

### STAFF

def aid(op: str, vaid: int) -> str:
    if op not in ["="]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "aid".')
    if not isinstance(vaid, (list, int)):
        raise InvalidTypeException(f'"{vaid}" is neither an integer nor a list. "aid" requires an integer or a list.')
    return f"aid{op}{vaid}"

### USER

def username(op: str, vusername: str) -> str:
    if op not in ["=", "!=", "~"]:
        raise InvalidOperatorException(f'"{op}" is not a valid Operator for "username".')
    if not isinstance(vusername, str):
        raise InvalidTypeException(f'"{vusername}" is not a string. "username" requires a string.')
    return f'username{op}"{vusername}"'
